@@ -64,7 +64,7 @@ class File(object):
     A class to configure redirection of stdin/stdout/stderr from/to a file.
     """
 
-    def __init__(self, path, append = False):
+    def __init__(self, path, append=False):
         self.path = path
         self.append = append
 
@@ -243,7 +243,7 @@ class Process:
         self.__command = []
 
         # Success status codes for this command
-        self.__ok_statuses = [ os.EX_OK ]
+        self.__ok_statuses = [os.EX_OK]
 
 
         # Process' pipes
@@ -282,11 +282,9 @@ class Process:
         if not self.__defer:
             self.execute()
 
-
     def __enter__(self):
         return self
 
-
     def __exit__(self, exc_type, exc_val, exc_tb):
         """
         Waits for the process termination and closes the output iterator (if
@@ -307,7 +305,6 @@ class Process:
 
         return False
 
-
     def __iter__(self):
         """Executes the process and returns an iterator to its output."""
 
@@ -315,7 +312,7 @@ class Process:
             self, self.__iter_raw, self.__iter_delimiter)
 
         try:
-            self._execute(stdout = iterator.pipe())
+            self._execute(stdout=iterator.pipe())
 
             with self.__lock:
                 self.__context_objects.append(weakref.ref(iterator))
@@ -325,7 +322,6 @@ class Process:
 
         return iterator
 
-
     def __or__(self, process):
         """Shell-style pipelines."""
 
@@ -352,14 +348,12 @@ class Process:
 
         return process
 
-
     def command(self):
         """Returns command arguments as it will be executed."""
 
         return self.__command[:]
 
-
-    def execute(self, wait = True, check_status = True):
+    def execute(self, wait=True, check_status=True):
         """Executes the command.
 
         :param wait: if :py:const:`True`, calls ``wait(check_status =
@@ -370,12 +364,11 @@ class Process:
         self._execute()
 
         if wait:
-            self.wait(check_status = check_status)
+            self.wait(check_status=check_status)
 
         return self
 
-
-    def kill(self, signal = signal.SIGTERM):
+    def kill(self, signal=signal.SIGTERM):
         """Kills the process.
 
         :param signal: signal which will be used to kill the process.
@@ -401,7 +394,6 @@ class Process:
 
         return False
 
-
     def pid(self):
         """Returns the process' PID."""
 
@@ -410,7 +402,6 @@ class Process:
 
         return self.__pid
 
-
     def raw_stderr(self):
         """
         Returns the process' captured raw stderr (if ``_stderr = PIPE``).
@@ -419,7 +410,6 @@ class Process:
         self.__ensure_terminated()
         return self.__stderr.getvalue()
 
-
     def raw_stdout(self):
         """
         Returns the process' captured raw stdout (if ``_stdout = PIPE``).
@@ -428,27 +418,23 @@ class Process:
         self.__ensure_terminated()
         return self.__stdout.getvalue()
 
-
     def status(self):
         """Returns the process' exit status."""
 
         self.__ensure_terminated()
         return self.__status
 
-
     def stderr(self):
         """Returns the process' captured stderr (if ``_stderr = PIPE``)."""
 
         return psys.u(self.raw_stderr())
 
-
     def stdout(self):
         """Returns the process' captured stdout (if ``_stdout = PIPE``)."""
 
         return psys.u(self.raw_stdout())
 
-
-    def wait(self, check_status = False, kill = None):
+    def wait(self, check_status=False, kill=None):
         """Waits for the process termination.
 
         :param check_status: if :py:const:`True`, check the process status
@@ -478,7 +464,7 @@ class Process:
 
         if self.__piped_from_process():
             self.__stdin_source.wait(
-                check_status = check_status, kill = kill)
+                check_status=check_status, kill=kill)
 
         if check_status:
             if self.__error is not None:
@@ -490,8 +476,7 @@ class Process:
 
         return self.__status
 
-
-    def _execute(self, stdout = None, check_pipes = True):
+    def _execute(self, stdout=None, check_pipes=True):
         """Executes the command."""
 
         if self.__on_execute is not None:
@@ -518,11 +503,10 @@ class Process:
                 self.__piped_from_process() and
                 self.__stdin_source._state() >= _PROCESS_STATE_RUNNING
             ):
-                self.__stdin_source.wait(kill = signal.SIGTERM)
+                self.__stdin_source.wait(kill=signal.SIGTERM)
 
             raise
 
-
     def _pipe_process(self, process):
         """Creates a pipe between two processes."""
 
@@ -530,13 +514,12 @@ class Process:
             if self.__state != _PROCESS_STATE_PENDING:
                 raise InvalidProcessState("Process can't be piped after execution")
 
-            if self.__stdin_source not in ( None, STDIN ):
+            if self.__stdin_source not in (None, STDIN):
                 raise InvalidOperation("The process' stdin is already redirected")
 
             LOG.debug("Creating a pipe: %s | %s", process, self)
             self.__stdin_source = process
 
-
     def _shell_command(self, stream, pipe_ok_statuses):
         """
         Generates a shell command which execution is equal to
@@ -549,6 +532,7 @@ class Process:
             arg = psys.b(arg)
 
             if simple_arg_re.search(arg) is None:
+                # Or probably we should use br"$'\x{0:02x}'".format(char)
                 stream.write(b"'" + arg.replace(b"'", b"""'"'"'""") + b"'")
             else:
                 stream.write(arg)
@@ -580,11 +564,11 @@ class Process:
             if isinstance(stdin_source, File):
                 stream.write(b" < ")
                 write_arg(stream, stdin_source.path)
-            elif stdin_source in ( None, STDIN ) or isinstance(stdin_source, Process):
+            elif stdin_source in (None, STDIN) or isinstance(stdin_source, Process):
                 pass
             elif (
-                type(stdin_source) in ( bytes, str ) or
-                isinstance(stdin_source, ( collections.Iterator, collections.Iterable ))
+                type(stdin_source) in (bytes, str) or
+                isinstance(stdin_source, (collections.Iterator, collections.Iterable))
             ):
                 raise InvalidOperation(
                     "String and iterator input is not supported for serialization to a shell script")
@@ -592,30 +576,35 @@ class Process:
                 raise LogicalError()
 
             # Stdout redirection
-            if self.__piped_to_process() or self.__stdout_target in ( STDOUT, PIPE ):
+            if self.__piped_to_process() or self.__stdout_target in (STDOUT, PIPE):
                 pass
             elif self.__stdout_target is STDERR:
                 stream.write(b" >&2")
             elif isinstance(self.__stdout_target, File):
-                stream.write(b" > ")
+                if self.__stdout_target.append:
+                    stream.write(b" >> ")
+                else:
+                    stream.write(b" > ")
                 write_arg(stream, self.__stdout_target.path)
             else:
                 raise LogicalError()
 
             # Stderr redirection
-            if self.__stderr_target in ( STDERR, PIPE ):
+            if self.__stderr_target in (STDERR, PIPE):
                 pass
             elif self.__stderr_target is STDOUT:
                 stream.write(b" 2>&1")
             elif isinstance(self.__stderr_target, File):
-                stream.write(b" 2> ")
+                if self.__stderr_target.append:
+                    stream.write(b" 2>> ")
+                else:
+                    stream.write(b" 2> ")
                 write_arg(stream, self.__stderr_target.path)
             else:
                 raise LogicalError()
 
             pipe_ok_statuses.append(self.__ok_statuses[:])
 
-
     def _shell_command_full(self):
         """
         Generates a shell command which execution is equal to self.execute()
@@ -643,13 +632,11 @@ class Process:
 
         return b"bash -c '" + command.getvalue().replace(b"'", b"""'"'"'""") + b"'"
 
-
     def _state(self):
         """Returns current process state."""
 
         return self.__state
 
-
     def __child(self):
         """Handles child process execution."""
 
@@ -665,11 +652,11 @@ class Process:
                     2: "stderr",
                 }
 
-                def redirect_fd(path, fd, write = True, append = False):
+                def redirect_fd(path, fd, write=True, append=False):
                     try:
                         if write:
                             file_fd = eintr_retry(os.open)(
-                                path, os.O_WRONLY | os.O_CREAT | ( os.O_APPEND if append else 0 ), 0o666)
+                                path, os.O_WRONLY | os.O_CREAT | (os.O_APPEND if append else os.O_TRUNC), 0o666)
                         else:
                             file_fd = eintr_retry(os.open)(path, os.O_RDONLY)
 
@@ -697,17 +684,17 @@ class Process:
 
                 # Configure stdin
                 if isinstance(self.__stdin_source, File):
-                    redirect_fd(self.__stdin_source.path, psys.STDIN_FILENO, write = False)
+                    redirect_fd(self.__stdin_source.path, psys.STDIN_FILENO, write=False)
 
                 # Configure stdout
                 if self.__stdout_target is STDERR:
                     try:
                         eintr_retry(os.dup2)(psys.STDERR_FILENO, psys.STDOUT_FILENO)
                     except Exception as e:
-                        raise Error("Unable to redirect stderr to stdout: {0}", psys.e(e))
+                        raise Error("Unable to redirect stdout to stderr: {0}", psys.e(e))
                 elif isinstance(self.__stdout_target, File):
                     redirect_fd(self.__stdout_target.path, psys.STDOUT_FILENO,
-                        append = self.__stdout_target.append)
+                        append=self.__stdout_target.append)
 
                 # Configure stderr
                 if self.__stderr_target is STDOUT:
@@ -717,10 +704,10 @@ class Process:
                         raise Error("Unable to redirect stderr to stdout: {0}", psys.e(e))
                 elif isinstance(self.__stderr_target, File):
                     redirect_fd(self.__stderr_target.path, psys.STDERR_FILENO,
-                        append = self.__stderr_target.append)
+                        append=self.__stderr_target.append)
 
                 # Required when we have C locale
-                command = [ psys.b(arg) for arg in self.__command ]
+                command = [psys.b(arg) for arg in self.__command]
 
                 exec_error = True
 
@@ -733,11 +720,10 @@ class Process:
                     exit_code = 126
 
                 print("Failed to execute '{program}': {error}.".format(
-                    program = self.__program, error = psys.e(e)), file = sys.stderr)
+                    program=self.__program, error=psys.e(e)), file=sys.stderr)
         finally:
             os._exit(exit_code)
 
-
     def __close(self):
         """
         Frees all allocated resources unneeded after the process termination
@@ -756,7 +742,6 @@ class Process:
             else:
                 self.__termination_fd = None
 
-
     def __communicate(self, poll):
         """Communicates with the process and waits for its termination."""
 
@@ -778,7 +763,7 @@ class Process:
             eintr_retry(fcntl.fcntl)(fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)
 
             poll.register(fd, poll.POLLIN if pipe.output else poll.POLLOUT)
-            pipe.close(read = not pipe.output, write = pipe.output)
+            pipe.close(read=not pipe.output, write=pipe.output)
             poll_objects += 1
         # Configure the poll object and pipes <--
 
@@ -808,7 +793,7 @@ class Process:
                             stdin = next(self.__stdin_generator)
 
                             try:
-                                if type(stdin) not in ( bytes, str ):
+                                if type(stdin) not in (bytes, str):
                                     raise TypeError("must be a string")
 
                                 stdin = psys.b(stdin)
@@ -841,7 +826,7 @@ class Process:
                             else:
                                 stdin = stdin[size:]
                 # stdout/stderr
-                elif pipe.source in ( psys.STDOUT_FILENO, psys.STDERR_FILENO ):
+                elif pipe.source in (psys.STDOUT_FILENO, psys.STDERR_FILENO):
                     data = eintr_retry(os.read)(fd, psys.BUFSIZE)
 
                     if data:
@@ -865,7 +850,7 @@ class Process:
 
             for pipe in self.__pipes:
                 if (
-                    pipe.source in ( psys.STDOUT_FILENO, psys.STDERR_FILENO ) and
+                    pipe.source in (psys.STDOUT_FILENO, psys.STDERR_FILENO) and
                     pipe.read is not None
                 ):
                     size = 0
@@ -891,7 +876,6 @@ class Process:
                             else:
                                 break
 
-
     def __communication_thread_func(self, fork_lock, poll):
         """A thread in which we communicate with the process."""
 
@@ -916,7 +900,6 @@ class Process:
         finally:
             poll.close()
 
-
     def __configure_stdio(self, stdout):
         """Configures the standard I/O file descriptors."""
 
@@ -931,14 +914,13 @@ class Process:
         elif self.__piped_from_process():
             # Connect and execute all processes in the pipe
 
-            pipe = Pipe(psys.STDIN_FILENO, output = False)
+            pipe = Pipe(psys.STDIN_FILENO, output=False)
             self.__pipes.append(pipe)
 
-            self.__stdin_source._execute(
-                stdout = pipe, check_pipes = False)
+            self.__stdin_source._execute(stdout=pipe, check_pipes=False)
         else:
-            if type(self.__stdin_source) in ( bytes, str ):
-                self.__stdin_generator = iter([ self.__stdin_source ])
+            if type(self.__stdin_source) in (bytes, str):
+                self.__stdin_generator = iter([self.__stdin_source])
             elif isinstance(self.__stdin_source, collections.Iterator):
                 self.__stdin_generator = self.__stdin_source
             elif isinstance(self.__stdin_source, collections.Iterable):
@@ -946,22 +928,22 @@ class Process:
             else:
                 raise LogicalError()
 
-            pipe = Pipe(psys.STDIN_FILENO, output = False)
+            pipe = Pipe(psys.STDIN_FILENO, output=False)
             self.__pipes.append(pipe)
         # stdin <--
 
         # stdout -->
-        if self.__stdout_target in ( STDOUT, STDERR ) or isinstance(self.__stdout_target, File):
+        if self.__stdout_target in (STDOUT, STDERR) or isinstance(self.__stdout_target, File):
             if stdout is not None:
                 raise LogicalError()
         elif self.__stdout_target is PIPE or self.__piped_to_process():
-            self.__pipes.append(Pipe(psys.STDOUT_FILENO, pipe = stdout))
+            self.__pipes.append(Pipe(psys.STDOUT_FILENO, pipe=stdout))
         else:
             raise LogicalError()
         # stdout <--
 
         # stderr -->
-        if self.__stderr_target in ( STDOUT, STDERR ) or isinstance(self.__stderr_target, File):
+        if self.__stderr_target in (STDOUT, STDERR) or isinstance(self.__stderr_target, File):
             pass
         elif self.__stderr_target is PIPE:
             self.__pipes.append(Pipe(psys.STDERR_FILENO))
@@ -969,14 +951,12 @@ class Process:
             raise LogicalError()
         # stdout <--
 
-
     def __ensure_terminated(self):
         """Ensures that the process is terminated."""
 
         if self.__state != _PROCESS_STATE_TERMINATED:
             raise InvalidProcessState("The process is not terminated")
 
-
     def __execute(self, stdout):
         """Executes the command."""
 
@@ -995,8 +975,8 @@ class Process:
 
             try:
                 self.__communication_thread = threading.Thread(
-                    target = self.__communication_thread_func,
-                    args = [ fork_lock, poll ])
+                    target=self.__communication_thread_func,
+                    args=(fork_lock, poll))
 
                 self.__communication_thread.daemon = True
                 self.__communication_thread.start()
@@ -1013,7 +993,7 @@ class Process:
 
             try:
                 self.__wait_thread = threading.Thread(
-                    target = self.__wait_pid_thread, args = [ fork_lock, termination_fd ])
+                    target=self.__wait_pid_thread, args=[fork_lock, termination_fd])
                 self.__wait_thread.daemon = True
                 self.__wait_thread.start()
             except BaseException as error:
@@ -1033,35 +1013,32 @@ class Process:
                 self.__child()
         # Fork the process <--
 
-
-    def __join_threads(self, timeout = None):
+    def __join_threads(self, timeout=None):
         """Joins all spawned threads."""
 
         if timeout is not None:
             end_time = time.time() + timeout
 
         if not psys.join_thread(self.__wait_thread,
-            timeout = None if timeout is None else end_time - time.time()):
+            timeout=None if timeout is None else end_time - time.time()):
             return False
 
         if not psys.join_thread(self.__communication_thread,
-            timeout = None if timeout is None else end_time - time.time()):
+            timeout=None if timeout is None else end_time - time.time()):
             return False
 
         return True
 
-
     def __on_output(self, fd, data):
         """Called when we got stdout/stderr data from the process."""
 
         (self.__stdout if fd == psys.STDOUT_FILENO else self.__stderr).write(data)
 
-
     def __parse_args(self, args, options):
         """Parses command arguments and options."""
 
         # Process options -->
-        def check_arg(option, value, types = tuple(), instance_of = tuple(), values = tuple()):
+        def check_arg(option, value, types=tuple(), instance_of=tuple(), values=tuple()):
             if type(value) not in types and not isinstance(value, instance_of) and value not in values:
                 raise InvalidArgument("Invalid value for option {0}", option)
 
@@ -1072,43 +1049,43 @@ class Process:
                 continue
 
             if option == "_defer":
-                self.__defer = check_arg(option, value, types = ( bool, ))
+                self.__defer = check_arg(option, value, types=(bool,))
             elif option == "_env":
                 if value is None:
                     self.__env = value
                 else:
                     self.__env = dict(
                         (
-                            psys.b(check_arg(option, k, types = ( bytes, str ))),
-                            psys.b(check_arg(option, v, types = ( bytes, str )))
+                            psys.b(check_arg(option, k, types=(bytes, str))),
+                            psys.b(check_arg(option, v, types=(bytes, str)))
                         )
-                            for k, v in value.items() )
+                        for k, v in value.items())
             elif option == "_iter_delimiter":
                 self.__iter_delimiter = psys.b(
-                    check_arg(option, value, types = ( bytes, str )))
+                    check_arg(option, value, types=(bytes, str)))
             elif option == "_iter_raw":
-                self.__iter_raw = check_arg(option, value, types = ( bool, ))
+                self.__iter_raw = check_arg(option, value, types=(bool,))
             elif option == "_ok_statuses":
                 self.__ok_statuses = [
-                    check_arg(option, status, types = ( int, )) for status in value ]
+                    check_arg(option, status, types=(int,)) for status in value]
             elif option == "_on_execute":
-                self.__on_execute = check_arg(option, value, instance_of = collections.Callable)
+                self.__on_execute = check_arg(option, value, instance_of=collections.Callable)
             elif option == "_shell":
-                self.__shell = check_arg(option, value, types = ( bool, ))
+                self.__shell = check_arg(option, value, types=(bool,))
             elif option == "_stderr":
                 self.__stderr_target = check_arg(
-                    option, value, instance_of = File, values = ( STDOUT, STDERR, PIPE ))
+                    option, value, instance_of=File, values=(STDOUT, STDERR, PIPE))
             elif option == "_stdin":
-                self.__stdin_source = check_arg(option, value, types = ( bytes, str ),
-                    instance_of = ( File, collections.Iterator, collections.Iterable ),
-                    values = ( STDIN, ))
+                self.__stdin_source = check_arg(option, value, types=(bytes, str),
+                    instance_of=(File, collections.Iterator, collections.Iterable),
+                    values=(STDIN,))
             elif option == "_stdout":
                 self.__stdout_target = check_arg(
-                    option, value, instance_of = File, values = ( STDOUT, STDERR, PIPE ))
+                    option, value, instance_of=File, values=(STDOUT, STDERR, PIPE))
             elif option == "_truncate_output":
-                self.__truncate_output = check_arg(option, value, types = ( bool, ))
+                self.__truncate_output = check_arg(option, value, types=(bool,))
             elif option == "_wait_for_output":
-                self.__wait_for_output = check_arg(option, value, types = ( bool, ))
+                self.__wait_for_output = check_arg(option, value, types=(bool,))
             else:
                 raise InvalidArgument("Invalid option: {0}", option)
         # Process options <--
@@ -1132,22 +1109,19 @@ class Process:
                     if value is not True:
                         self.__command.append(_get_arg_value(value, self.__shell))
 
-        self.__command += [ _get_arg_value(arg, self.__shell) for arg in args ]
+        self.__command += [_get_arg_value(arg, self.__shell) for arg in args]
         # Command arguments <--
 
-
     def __piped_from_process(self):
         """Returns True if this process is piped from another process."""
 
         return isinstance(self.__stdin_source, Process)
 
-
     def __piped_to_process(self):
         """Returns True if this process is piped to another process."""
 
         return isinstance(self.__stdout_target, Process)
 
-
     def __to_bytes(self):
         """Returns the command string.
 
@@ -1157,7 +1131,6 @@ class Process:
 
         return psys.b(self.__to_str())
 
-
     def __to_str(self):
         """Returns the command string.
 
@@ -1171,35 +1144,28 @@ class Process:
             if command:
                 command += " "
 
-            if type(arg) == bytes:
-                for c in b""" '"\\\r\n\t""":
-                    if c in arg:
-                        arg = repr(arg)[1:] if PY3 else repr(arg)
-                        break
-                else:
-                    arg = repr(arg)[1 + int(PY3):-1]
+            command += _arg_to_str(arg)
 
-                command += psys.u(arg)
+        if self.__stdout_target is STDERR:
+            command += " >&2"
+        elif isinstance(self.__stdout_target, File):
+            if self.__stdout_target.append:
+                command += " >> "
             else:
-                for c in """ '"\\\r\n\t""":
-                    if c in arg:
-                        for replacement in (
-                            ( "\\", r"\\" ),
-                            ( "'",  r"\'" ),
-                            ( "\r", r"\r" ),
-                            ( "\n", r"\n" ),
-                            ( "\t", r"\t" ),
-                        ):
-                            arg = arg.replace(*replacement)
+                command += " > "
+            command += _arg_to_str(self.__stdout_target.path)
 
-                        arg = "'" + arg + "'"
-                        break
-
-                command += arg
+        if self.__stderr_target is STDOUT:
+            command += " 2>&1"
+        elif isinstance(self.__stderr_target, File):
+            if self.__stderr_target.append:
+                command += " 2>> "
+            else:
+                command += " 2> "
+            command += _arg_to_str(self.__stderr_target.path)
 
         return command
 
-
     def __wait_pid_thread(self, fork_lock, termination_fd):
         """Waits for the process termination."""
 
@@ -1236,7 +1202,6 @@ class Process:
             except Exception as e:
                 LOG.error("Unable to close a pipe: %s.", psys.e(e))
 
-
     if PY3:
         __bytes__ = __to_bytes
         __str__ = __to_str
@@ -1245,15 +1210,50 @@ class Process:
         __unicode__ = __to_str
 
 
-
 def _get_arg_value(value, shell):
     """Returns an argument string value."""
 
-    if type(value) in ( bytes, str ):
+    if type(value) in (bytes, str):
         return value
-    elif type(value) in ( int, float ) + ( tuple() if PY3 else (long,) ):
+    elif type(value) in (int, float) + (tuple() if PY3 else (long,)):
         return str(value)
     elif shell and isinstance(value, Process):
         return value._shell_command_full()
     else:
         raise InvalidArgument("Invalid argument: command arguments must be basic types only")
+
+
+def _arg_to_str(arg):
+    """
+    Returns a string representation (possibly quoted)
+    of a command argument as it would be used in shell scripts.
+
+    .. note::
+        Very lazy formatting. Not suitable for automatic generation of shell scripts.
+    """
+
+    if type(arg) == bytes:
+        for c in b""" '"\\\r\n\t""":
+            if c in arg:
+                arg = repr(arg)[1:] if PY3 else repr(arg)
+                break
+        else:
+            arg = repr(arg)[1 + int(PY3):-1]
+
+        return psys.u(arg)
+    else:
+        for c in """ '"\\\r\n\t""":
+            if c in arg:
+                for replacement in (
+                    ("\\", r"\\"),
+                    ("'",  r"\'"),
+                    ("\r", r"\r"),
+                    ("\n", r"\n"),
+                    ("\t", r"\t"),
+                ):
+                    arg = arg.replace(*replacement)
+
+                arg = "'" + arg + "'"
+                break
+
+        return arg
